# Files Map

# Extenions:        .yaml, .json, .csv., .xlsx, .png
# Sub-extensions:   .Topic.yaml, .Comment.yaml, .Drawing.png

# In order to hard-assign files with sub-extensions, we need a way to link sub-extensions with Wikis for concepts.

# Example:
'''
*._topic#metaculus.*         <=> _:topic#metaculus
*.__mindey@topic#metaculus.* <=> ::mindey/topic#metaculus
*.WD@Q123.*                  <=> WD:Q123
'''

FMAP = {}

def f2n(token):
    'filename to name'

    # PARSE
    if '#' in token:
        _concept, _format = token.rsplit('#', 1)
    else:
        _concept, _format = token, None

    if '@' in _concept:
        _namespace, _alias = _concept.rsplit('@', 1)
    else:
        _namespace, _alias = _concept, None

    # MERGE
    name = ''

    if _namespace.startswith('__'):     # github wikis
        name += '::'+_namespace[2:]
    elif _namespace.startswith('_'):
        name += '_:'+_namespace[1:]     # infamily wiki
    elif _namespace == 'WD':            # wikidata
        name += _namespace + ':'

    if _alias is not None:
        if _namespace.startswith('__'): # github wikis
            name += '/'
        name += _alias

    if _format is not None:
        name += '#'+_format

    return name



def n2f(token):
    'name to filename'

    # PARSE
    if '#' in token:
        _concept, _format = token.rsplit('#', 1)
    else:
        _concept, _format = token, None

    if '/' in _concept:
        _namespace, _alias = _concept.rsplit('/', 1)
    elif ':' in _concept:
        _namespace, _alias = _concept.rsplit(':', 1)
#       _namespace += ':'
    else:
        _namespace, _alias = _concept, None

    # MERGE
    name = ''

    if _namespace == '_':               # infamily
        name += _namespace
    elif _namespace.startswith('::'):
        name += '__' + _namespace[2:] + '@'   # infamily wiki
    elif _namespace == 'WD':            # wikidata
        name += _namespace + '@'

    if _alias is not None:
        if _namespace.startswith('__'): # github wikis
            name += '/'
        name += _alias

    if _format is not None:
        name += '#'+_format

    return name



NAMESPACES = {}

# Infinity namespace, to be defined on
# https://github.com/infamily/ooio/wiki
# and refered by keys starting with '_:'
INFINITY = {
    '_:{concept}': [
        'https://github.com/infamily/_/wiki/{concept}',
        'https://raw.githubusercontent.com/wiki/infamily/_/{concept}.md'
    ],
    '_:{concept}#{format}': [
        'https://github.com/infamily/_/wiki/{concept}#{format}',
        'https://raw.githubusercontent.com/wiki/infamily/_/{concept}.md#{format}'
    ]
}
NAMESPACES['_:'] = INFINITY


# Wikidata namespace, to be defined on
# https://www.wikidata.org
# and refered by keys starting with 'WD:'
WIKIDATA = lambda N: {
    '%s:Q{integer}' % (N,): [
        'https://www.wikidata.org/wiki/Q{integer}'
    ],
    '%s:P{integer}' % (N,): [
        'https://www.wikidata.org/wiki/Property:P{integer}'
    ]
}
NAMESPACES['WD:'] = WIKIDATA('WD')


# Github ooio wiki namespace, to be defined on M repo wikis
# https://github.com/{user}/M/wiki/{concept}
# and refered by keys starting with 'N:{user}/{concept}'
GITHUB_WIKI = lambda N, M: {
    '%s:{user}/{concept}' % (N,): [
        'https://github.com/{user}/%s/wiki/{concept}' % (M,),
        'https://raw.githubusercontent.com/wiki/{user}/%s/{concept}.md' % (M,),
    ],
    '%s:{user}/{concept}#{format}' % (N,): [
        'https://github.com/{user}/%s/wiki/{concept}#{format}' % (M,),
        'https://raw.githubusercontent.com/wiki/{user}/%s/{concept}.md#{format}' % (M,),
    ]
}
NAMESPACES['IN:'] = GITHUB_WIKI('IN', 'ooio') # WIKIs of ooio repos
NAMESPACES['@:'] = GITHUB_WIKI('@', 'terms') # WIKIs of terms repos
NAMESPACES['::'] = GITHUB_WIKI(':', '-') # WIKIs of - repos

# Files of ooio repos namespaces, to be defined on M repo files
# https://github.com/{user}/M/wiki/{concept}
# and refered by keys starting with 'N:{user}/{filename}'
GITHUB_FILE = lambda N, M: {
    '%s:{user}/{concept}' % (N,): [
        'https://github.com/{user}/%s/blob/master/{concept}' % (M,),
        'https://raw.githubusercontent.com/{user}/%s/{concept}' % (M,),
    ],
    '%s:{user}/{concept}#{format}' % (N,): [
        'https://github.com/{user}/%s/blob/master/{concept}#{format}' % (M,),
        'https://raw.githubusercontent.com/{user}/%s/{concept}#{format}' % (M,),
    ]
}
NAMESPACES['FN:'] = GITHUB_FILE('FN', 'ooio') # FILEs of ooio repos
NAMESPACES['-:'] = GITHUB_FILE('-', '-') # FILEs of ooio repos


MAP = {}

for key, value in NAMESPACES.items():
    MAP = dict(MAP, **value)



from .fmap import (
    n2f,
    f2n
)

def test_f2n():

    # _:
    assert f2n('_topic#metaculus') == '_:topic#metaculus'

    # ::
    assert f2n('__mindey@topic#metaculus') == '::mindey/topic#metaculus'

    # WD
    assert f2n('WD@Q123') == 'WD:Q123'

def test_n2f():

    # _:
    assert n2f('_:topic#metaculus') == '_topic#metaculus'

    # ::
    assert n2f('::mindey/topic#metaculus') == '__mindey@topic#metaculus'

    # WD
    assert n2f('WD:Q123') == 'WD@Q123'



from metawiki import (
    url_to_name,
    name_to_url
)

def test_to_name():

    # _:
    result = url_to_name(
        'https://github.com/infamily/_/wiki/percent-change#1hourly')

    assert result == '_:percent-change#1hourly'

    # IN:
    result = url_to_name(
        'https://github.com/infamily/ooio/wiki/percent-change#1hourly')

    assert result == 'IN:infamily/percent-change#1hourly'

    # @:
    result = url_to_name(
        'https://github.com/infamily/terms/wiki/percent-change#1hourly')

    assert result == '@:infamily/percent-change#1hourly'

    # ::
    result = url_to_name(
        'https://github.com/infamily/-/wiki/percent-change#1hourly')

    assert result == '::infamily/percent-change#1hourly'

    # FN:
    result = url_to_name(
        'https://github.com/mindey/ooio/blob/master/keys.md#a225d1e4')

    assert result == 'FN:mindey/keys.md#a225d1e4'

    # -:
    result = url_to_name(
        'https://github.com/mindey/-/blob/master/keys.md#a225d1e4')

    assert result == '-:mindey/keys.md#a225d1e4'


    # WD:
    result =  url_to_name('https://www.wikidata.org/wiki/Q1347367')

    assert result == 'WD:Q1347367'

    result =  url_to_name('https://www.wikidata.org/wiki/Property:P31')

    assert result == 'WD:P31'

def test_to_url():

    # _:
    result = name_to_url('_:level')

    assert result == 'https://github.com/infamily/_/wiki/level'

    # IN:
    result = name_to_url('IN:infamily/level')

    assert result == 'https://github.com/infamily/ooio/wiki/level'

    # @:
    result = name_to_url('@:infamily/level')

    assert result == 'https://github.com/infamily/terms/wiki/level'

    # ::
    result = name_to_url('::infamily/level')

    assert result == 'https://github.com/infamily/-/wiki/level'

    # FN:
    result = name_to_url('FN:mindey/keys.md#a225d1e4')

    assert result == 'https://github.com/mindey/ooio/blob/master/keys.md#a225d1e4'

    # -:
    result = name_to_url('-:mindey/keys.md#a225d1e4')

    assert result == 'https://github.com/mindey/-/blob/master/keys.md#a225d1e4'

    # WD:
    result = name_to_url('WD:Q1347367')

    assert result == 'https://www.wikidata.org/wiki/Q1347367'



from metawiki.map import MAP, NAMESPACES
from metawiki.fmap import f2n, n2f

import parse

class MetaWikiError(Exception):
    pass


urlspaces = {
    'http://': 'HTTP location',
    'https://': 'HTTPS location'
}

def isname(string):
    return any([string.startswith(name) for name in NAMESPACES])

def isurl(string):
    return any([string.startswith(url) for url in urlspaces])


def convert(name, template, inverse=False):

    if inverse:
        REVERSED_MAP = {item[-1][0]: [item[0]] for item in MAP.items()}
        endplate = REVERSED_MAP[template][0]
    else:
        endplate = MAP[template][0]

    p = parse.parse(template, name)

    return endplate.format(**p.named)


def name_to_url(name, skip_valid=True):

    if skip_valid:
        if isurl(name):
            return name

    template = None

    # _:
    if name.startswith('_:'):
        if not '#' in name:
            template = '_:{concept}'
        else:
            template = '_:{concept}#{format}'

    # IN:
    if name.startswith('IN:'):
        if not '#' in name:
            template = 'IN:{user}/{concept}'
        else:
            template = 'IN:{user}/{concept}#{format}'

    # @:
    if name.startswith('@:'):
        if not '#' in name:
            template = '@:{user}/{concept}'
        else:
            template = '@:{user}/{concept}#{format}'

    # ::
    if name.startswith('::'):
        if not '#' in name:
            template = '::{user}/{concept}'
        else:
            template = '::{user}/{concept}#{format}'


    # FN:
    if name.startswith('FN:'):
        if not '#' in name:
            template = 'FN:{user}/{concept}'
        else:
            template = 'FN:{user}/{concept}#{format}'

    # -:
    if name.startswith('-:'):
        if not '#' in name:
            template = '-:{user}/{concept}'
        else:
            template = '-:{user}/{concept}#{format}'

    # WD:
    elif name.startswith('WD:'):
        if name.startswith('WD:Q'):
            template = 'WD:Q{integer}'

        elif name.startswith('WD:P'):
            template = 'WD:P{integer}'

        else:
            raise MetaWikiError(
                "Wikidata concept cannot start with anything other than WD:Q or WD:P, you provided: {}.".format(name)
            )

    if template:
        return convert(name, template)
    else:
        raise MetaWikiError(
            "Could not find the template for concept: {}.".format(name)
        )


def url_to_name(url, skip_valid=True):

    if skip_valid:
        if isname(url):
            return url

    template = None

    # _:, IN:
    if url.startswith('https://github.com/'):
        # _:
        if '/infamily/_/wiki/' in url:
            if not '#' in url:
                template = 'https://github.com/infamily/_/wiki/{concept}'
            else:
                template = 'https://github.com/infamily/_/wiki/{concept}#{format}'
        # IN:
        elif '/ooio/wiki/' in url:
            if not '#' in url:
                template = 'https://github.com/{user}/ooio/wiki/{concept}'
            else:
                template = 'https://github.com/{user}/ooio/wiki/{concept}#{format}'

        # @:
        elif '/terms/wiki/' in url:
            if not '#' in url:
                template = 'https://github.com/{user}/terms/wiki/{concept}'
            else:
                template = 'https://github.com/{user}/terms/wiki/{concept}#{format}'

        # ::
        elif '/-/wiki/' in url:
            if not '#' in url:
                template = 'https://github.com/{user}/-/wiki/{concept}'
            else:
                template = 'https://github.com/{user}/-/wiki/{concept}#{format}'

        # -:
        elif '/-/blob/master/' in url:
            if not '#' in url:
                template = 'https://github.com/{user}/-/blob/master/{concept}'
            else:
                template = 'https://github.com/{user}/-/blob/master/{concept}#{format}'

        # FN:
        elif '/ooio/blob/master/' in url:
            if not '#' in url:
                template = 'https://github.com/{user}/ooio/blob/master/{concept}'
            else:
                template = 'https://github.com/{user}/ooio/blob/master/{concept}#{format}'

        else:
            raise MetaWikiError("Undefined GitHub namespace for: {}. (currently, only repo=_ in url are valid)".format(url))

    # WD:
    if url.startswith('https://www.wikidata.org/'):
        if '/wiki/Q' in url:
            template = 'https://www.wikidata.org/wiki/Q{integer}'
        elif '/wiki/Property:P' in url:
            template = 'https://www.wikidata.org/wiki/Property:P{integer}'
        else:
            raise MetaWikiError("Undefined Wikidata namespace for: {}.".format(url))

    if template:
        return convert(url, template, inverse=True)
    else:
        raise MetaWikiError(
            "Could not find the template for url: {}.".format(url)
        )

def ext2url(token, skip_valid=True):
    return name_to_url(f2n(token), skip_valid=skip_valid)

def url2ext(url, skip_valid=True):
    return n2f(url_to_name(url, skip_valid=skip_valid))
